/*
 * Copyright 2006, Jérôme Duval. All rights reserved.
 * Distributed under the terms of the MIT License.
 */

#include <pthread.h>
#include <signal.h>
#include <stdlib.h>
#include "pthread_private.h"

static const pthread_attr pthread_attr_default = {
	PTHREAD_CREATE_JOINABLE,
	B_NORMAL_PRIORITY
};

struct pthread_data {
	thread_entry entry;
	void *data;
};

static int32
_pthread_thread_entry(void *entry_data)
{
	struct pthread_data *pdata = (struct pthread_data *)entry_data;
	void *(*entry)(void*) = (void *(*)(void*))pdata->entry;
	void *data = pdata->data;

	free(pdata);
	on_exit_thread(_pthread_key_call_destructors, NULL);
	
	pthread_exit(entry(data));
	return B_OK;
}


int 
pthread_create(pthread_t *_thread, const pthread_attr_t *_attr,
		void *(*start_routine)(void*), void *arg)
{
	thread_id thread;
	const pthread_attr *attr = NULL;
	struct pthread_data *pdata;

	if (_thread == NULL)
		return B_BAD_VALUE;

	if (_attr == NULL)
		attr = &pthread_attr_default;
	else
		attr = *_attr;
	
	pdata = malloc(sizeof(struct pthread_data));
	if (!pdata)
		return B_WOULD_BLOCK;
	pdata->entry = (thread_entry)start_routine;
	pdata->data = arg;

	thread = spawn_thread(_pthread_thread_entry, "pthread func", attr->sched_priority, pdata);
	if (thread < B_OK)
		return B_WOULD_BLOCK;
		// stupid error code (EAGAIN) but demanded by POSIX

	*_thread = thread;
	resume_thread(thread);

	return B_OK;
}


pthread_t
pthread_self()
{
	return find_thread(NULL);
}


int 
pthread_equal(pthread_t t1, pthread_t t2)
{
	return (t1 > 0 && t2 > 0 && t1 == t2);
}

int 
pthread_join(pthread_t thread, void **value_ptr)
{
	wait_for_thread(thread, (status_t *)value_ptr);
	/* the thread could be joinable and gone */
	return B_OK;
}


int
pthread_once(pthread_once_t *once_control, void (*init_routine)(void))
{
	return B_UNSUPPORTED;
}


void 
pthread_exit(void *value_ptr)
{
	exit_thread((status_t)value_ptr);
}


int 
pthread_kill(pthread_t thread, int sig)
{
	status_t err =  kill(thread, sig);
	if (err == B_BAD_THREAD_ID)
		return ESRCH;
	return err;
}


int
pthread_setcancelstate(int state, int *old_state)
{
	return B_UNSUPPORTED;
}


int
pthread_setcanceltype(int type, int *old_type)
{
	return B_UNSUPPORTED;
}


int
pthread_testcancel(void)
{
	return B_UNSUPPORTED;
}


int
pthread_cleanup_push(void (*routine)(void *), void *arg)
{
	return B_UNSUPPORTED;
}


int
pthread_cleanup_pop(int execute)
{
	return B_UNSUPPORTED;
}


int
pthread_cancel(pthread_t thread)
{
	// ToDo: do properly
	return pthread_kill(thread, 0);
}


int 
pthread_detach(pthread_t thread)
{
	return B_NOT_ALLOWED;
}

